;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.2 - GPL3 - Apr 2012
;
;-----------------------------------------------------------------------------------
; Misc Functions
;-----------------------------------------------------------------------------------

;os_get_timecounter
;os_delay
;os_seed_random
;os_get_random
;os_get_random_integer
;os_get_argc
;os_get_argv
;os_get_strv


; -----------------------------------------------------------------------------
; os_get_timecounter -- Get the current RTC clock counter value
; IN:	Nothing
; OUT:	RAX = Time in eights of a second since clock started
; This function depends on the RTC (IRQ 8) so interrupts must be enabled.
os_get_timecounter:
	mov rax, [os_ClockCounter]		; Grab the timer counter value. It increments 8 times a second
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_delay -- Delay by X eights of a second
; IN:	RAX = Time in eights of a second
; OUT:	All registers preserved
; A value of 8 in RAX will delay 1 second and a value of 1 will delay 1/8 of a second
; This function depends on the RTC (IRQ 8) so interrupts must be enabled.
os_delay:
	push rcx
	push rax

	mov rcx, [os_ClockCounter]		; Grab the initial timer counter. It increments 8 times a second
	add rax, rcx				; Add RCX so we get the end time we want
os_delay_loop:
	cmp qword [os_ClockCounter], rax	; Compare it against our end time
	jbe os_delay_loop			; Loop if RAX is still lower

	pop rax
	pop rcx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_seed_random -- Seed the RNG based on the current date and time
; IN:	Nothing
; OUT:	All registers preserved
os_seed_random:
	push rdx
	push rbx
	push rax

	xor rbx, rbx
	mov al, 0x09		; year
	out 0x70, al
	in al, 0x71
	mov bl, al
	shl rbx, 8
	mov al, 0x08		; month
	out 0x70, al
	in al, 0x71
	mov bl, al
	shl rbx, 8
	mov al, 0x07		; day
	out 0x70, al
	in al, 0x71
	mov bl, al
	shl rbx, 8
	mov al, 0x04		; hour
	out 0x70, al
	in al, 0x71
	mov bl, al
	shl rbx, 8
	mov al, 0x02		; minute
	out 0x70, al
	in al, 0x71
	mov bl, al
	shl rbx, 8
	mov al, 0x00		; second
	out 0x70, al
	in al, 0x71
	mov bl, al
	shl rbx, 16
	rdtsc			; Read the Time Stamp Counter in EDX:EAX
	mov bx, ax		; Only use the last 2 bytes

	mov [os_RandomSeed], rbx	; Seed will be something like 0x091229164435F30A

	pop rax
	pop rbx
	pop rdx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_get_random -- Return a random integer
; IN:	Nothing
; OUT:	RAX = Random number
;	All other registers preserved
os_get_random:
	push rdx
	push rbx

	mov rax, [os_RandomSeed]
	mov rdx, 0x23D8AD1401DE7383	; The magic number (random.org)
	mul rdx				; RDX:RAX = RAX * RDX
	mov [os_RandomSeed], rax

	pop rbx
	pop rdx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_get_random_integer -- Return a random integer between Low and High (incl)
; IN:	RAX = Low integer
;	RBX = High integer
; OUT:	RCX = Random integer
os_get_random_integer:
	push rdx
	push rbx
	push rax

	sub rbx, rax		; We want to look for a number between 0 and (High-Low)
	call os_get_random
	mov rdx, rbx
	add rdx, 1
	mul rdx
	mov rcx, rdx

	pop rax
	pop rbx
	pop rdx
	add rcx, rax		; Add the low offset back
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_get_argc -- Return the number arguments passed to the program (RUN string)
; IN:	Nothing
; OUT:	AL = Number of arguments
os_get_argc:
	xor eax, eax
	mov al, [prg_args]
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_get_argv -- Get the value of an argument that was passed to the program
;		 arg[0], arg[1], arg[2], ...			 ( 0 = RUN )
; IN:	AL = Argument number
; OUT:	RSI = Start of numbered argument string
os_get_argv:
	push rcx
	push rax
	mov rsi, prg_path_args_string
	cmp al, 0x00
	je os_get_argv_end
	mov cl, al

os_get_argv_nextchar:
	lodsb
	cmp al, 0x00
	jne os_get_argv_nextchar
	dec cl
	cmp cl, 0
	jne os_get_argv_nextchar
	
os_get_argv_end:
	pop rax
	pop rcx
	ret
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; os_get_strv -- Get the value from a string to parse
;		 value[0], value[1], value[2], ...
; IN:	AL  = Value number
;	AH  = Delimiter
;	RSI = String
; OUT:	RSI = Start of numbered value string
os_get_strv:
	push rcx
	push rax
	
	cmp al, 0
	je os_get_strv_end
	mov cl, al

os_get_strv_nextchar:
	lodsb
	cmp al, ah
	jne os_get_strv_nextchar
	dec cl
	cmp cl, 0
	jne os_get_strv_nextchar
	
os_get_strv_end:
	pop rax
	pop rcx
	ret
; -----------------------------------------------------------------------------


; =============================================================================
; EOF
